// Generated by CoffeeScript 1.10.0
(function() {
  var ListItem, React, div, immutable;

  immutable = require('immutable');

  React = require('react');

  div = React.DOM.div;

  ListItem = React.createFactory(require('./list-item'));

  module.exports = React.createClass({
    displayName: 'ListView',
    propTypes: {
      items: React.PropTypes.object.isRequired,
      selectedItemId: React.PropTypes.oneOfType([React.PropTypes.number, React.PropTypes.string]),
      collapsedItemIds: React.PropTypes.object.isRequired,
      itemClassName: React.PropTypes.string,
      selectedItemClassName: React.PropTypes.string,
      ignoreCollapseClicks: React.PropTypes.bool,
      handler: React.PropTypes.func,
      renderItem: React.PropTypes.func,
      onCollapseItem: React.PropTypes.func.isRequired,
      onExpandItem: React.PropTypes.func.isRequired,
      onSelectItem: React.PropTypes.func.isRequired
    },
    getDefaultProps: function() {
      return {
        itemClassName: '',
        selectedItemClassName: 'selected',
        ignoreCollapseClicks: false
      };
    },
    getInitialState: function() {
      return this._getState(this.props);
    },
    componentWillReceiveProps: function(nextProps) {
      return this.setState(this._getState(nextProps));
    },
    _getState: function(props) {
      return {
        selectedItemId: props.selectedItemId,
        collapsedItemIds: immutable.Set(props.collapsedItemIds)
      };
    },
    _getItemById: function(itemId) {
      var foundItem;
      foundItem = null;
      this.props.items.forEach(function(item) {
        var ref;
        if (item.id === itemId) {
          return foundItem = item;
        } else if (((ref = item.children) != null ? ref.size : void 0) > 0) {
          return item.children.find(function(subItem) {
            if (subItem.id === itemId) {
              return foundItem = subItem;
            }
          });
        }
      });
      return foundItem;
    },
    _findSelectedItemPath: function(items) {
      var path;
      path = null;
      items.forEach((function(_this) {
        return function(item, index) {
          var ref, subItemPath;
          if (item.id === _this.state.selectedItemId) {
            return path = [index];
          } else if (((ref = item.children) != null ? ref.size : void 0) > 0 && !_this.state.collapsedItemIds.contains(item.id)) {
            subItemPath = _this._findSelectedItemPath(item.children);
            if (subItemPath) {
              return path = [index, subItemPath[0]];
            }
          }
        };
      })(this));
      return path;
    },
    _normalizeParentStep: function(step) {
      return Math.sign(step);
    },
    _isUnfoldedParent: function(item) {
      var isFolded, isParent, ref;
      isParent = ((ref = item.children) != null ? ref.size : void 0) > 0;
      isFolded = this.state.collapsedItemIds.contains(item.id);
      return isParent && !isFolded;
    },
    _updateSelectedItemId: function(newItem) {
      return this.props.onSelectItem(newItem.id, newItem);
    },
    _moveSelection: function(step) {
      var candidate, candidateParent, candidates, itemIndex, parentItemIndex, parentStep, selectedItem, selectedItemPath;
      selectedItem = this._getItemById(this.state.selectedItemId);
      selectedItemPath = this._findSelectedItemPath(this.props.items);
      if (this._isUnfoldedParent(selectedItem) && this._normalizeParentStep(step) === 1) {
        return this._updateSelectedItemId(selectedItem.children.first());
      } else if ((selectedItemPath != null ? selectedItemPath.length : void 0) === 1) {
        itemIndex = (selectedItemPath[0] + Number(step)) % this.props.items.size;
        candidate = this.props.items.get(itemIndex);
        if (this._isUnfoldedParent(candidate) && this._normalizeParentStep(step) === -1) {
          return this._updateSelectedItemId(candidate.children.last());
        } else if (candidate) {
          return this._updateSelectedItemId(candidate);
        } else {
          throw new Error('bad top-level selectedItemPath');
        }
      } else if ((selectedItemPath != null ? selectedItemPath.length : void 0) > 1) {
        candidateParent = this.props.items.get(selectedItemPath[0]);
        candidates = candidateParent.children;
        itemIndex = selectedItemPath[1] + Number(step);
        candidate = candidates.get(itemIndex);
        if (itemIndex === -1) {
          return this._updateSelectedItemId(candidateParent);
        } else if (candidate) {
          return this._updateSelectedItemId(candidate);
        } else {
          parentStep = this._normalizeParentStep(step);
          parentItemIndex = (selectedItemPath[0] + parentStep) % this.props.items.size;
          candidate = this.props.items.get(parentItemIndex);
          return this._updateSelectedItemId(candidate);
        }
      } else {
        throw new Error('bad selectedItemPath');
      }
    },
    _expandSelection: function() {
      var ref;
      if (((ref = this._getItemById(this.state.selectedItemId).children) != null ? ref.size : void 0) > 0) {
        return this.props.onExpandItem(this.state.selectedItemId);
      }
    },
    _collapseSelection: function() {
      var ref;
      if (((ref = this._getItemById(this.state.selectedItemId).children) != null ? ref.size : void 0) > 0) {
        return this.props.onCollapseItem(this.state.selectedItemId);
      }
    },
    _handleKeyPress: function(e) {
      return this._handleShortcut(e.key, e);
    },
    _handleShortcut: function(type, event) {
      var action;
      action = this.props.handler(type, event);
      if (action != null ? action.moveSelection : void 0) {
        this._moveSelection(action.moveSelection);
        return event.stopPropagation();
      } else if (action != null ? action.collapseSelection : void 0) {
        this._collapseSelection();
        return event.stopPropagation();
      } else if (action != null ? action.expandSelection : void 0) {
        this._expandSelection();
        return event.stopPropagation();
      }
    },
    _handleClickRequest: function(itemId, item) {
      this.props.onSelectItem(itemId, item);
      if (this.props.ignoreCollapseClicks) {
        return;
      }
      if (this.state.collapsedItemIds.contains(itemId)) {
        return this.props.onExpandItem(itemId);
      } else {
        return this.props.onCollapseItem(itemId);
      }
    },
    _renderListItem: function(item, subListPath) {
      return ListItem({
        item: item,
        key: item.id + "_" + (subListPath.join('')),
        className: this.props.itemClassName,
        selected: item.id === this.state.selectedItemId,
        selectedClassName: this.props.selectedItemClassName,
        onClickRequest: this._handleClickRequest
      }, this.props.renderItem(item.id, subListPath.toJS(), item));
    },
    _renderSubList: function(items, subListPath) {
      var renderedList;
      if (subListPath == null) {
        subListPath = null;
      }
      if (!subListPath) {
        subListPath = immutable.List();
      }
      renderedList = immutable.List();
      items.forEach((function(_this) {
        return function(item, index) {
          var ref;
          if (((ref = item.children) != null ? ref.size : void 0) > 0) {
            renderedList = renderedList.push(_this._renderListItem(item, subListPath));
            if (!_this.state.collapsedItemIds.contains(item.id)) {
              return renderedList = renderedList.concat(_this._renderSubList(item.children, subListPath.push(item.id)));
            }
          } else {
            return renderedList = renderedList.push(_this._renderListItem(item, subListPath));
          }
        };
      })(this));
      return renderedList;
    },
    render: function() {
      return div({
        tabIndex: -1,
        className: this.props.className,
        onKeyDown: this._handleKeyPress
      }, this._renderSubList(this.props.items));
    }
  });

}).call(this);
